@@ -19,35 +19,94 @@ Firewall::PaloAlto - Interact with a Palo Alto firewall's API through Perl.
 
 =head1 VERSION
 
-Version 0.01
+Version 0.02
 
 =cut
 
-our $VERSION = '0.01';
-
+our $VERSION = '0.02';
 
 =head1 SYNOPSIS
 
-Quick summary of what the module does.
-
-Perhaps a little code snippet.
+The Firewall::PaloAlto module provides interfaces into the XML API of a Palo Alto firewall.
 
     use Firewall::PaloAlto;
 
-    my $foo = Firewall::PaloAlto->new();
-    ...
+    my $fw = Firewall::PaloAlto->new(host => 'pa.local', username => 'admin', password => 'admin');
+    $fw->connect();
 
-=head1 EXPORT
+    #Add a new virtual system
+    $fw->address('set', vsys_id => 6, 'display-name' => "Script_Tenant");
+
+    #Add a virtual router to the chassis
+    $fw->virtual_router('set', vr_name => 'NEW_VR', interface => [ 'ae1.65', 'ae1.66' ]);
+
+    #Add a new address - if the vsys is not specified it defaults to vsys1.
+    #This works for devices such as the VM series, which only have a vsys1. 
+    $fw->address('set', name => 'Google_DNS', ip-netmask => '8.8.8.8/32');
+
+    #Get the configuration for the newly created address:
+    my $address_config = $fw->address('get', name => 'Google_DNS');
+
+    #Delete the newly created address
+    $fw->address('delete', name => 'Google_DNS);
 
 A list of functions that can be exported.  You can delete this section
 if you don't export anything, such as for a purely object-oriented module.
 
 =cut
 
+=head1 CLASS METHODS
+
+=head2 connect(%parameters)
+
+This function connects to the Palo Alto with the credentials specified.
+
+Host, username and password are mandatory.
+If not specified SSL is used, but it can be disabled using the argument ssl => 0
+
+Detailed debugging can be turned on using the debu => 1 argument. It is off by default.
+
+=head3 Arguments
+
+=over 
+
+=item * 
+host - the hostname or IP of the firewall to connect to.
+
+=item *
+username - a username to connect to the firewall.
+
+=item *
+password - a password to connect to the firewall.
+
+=item *
+ssl (optional, default: 1) - use SSL to connect to the firewall. 
+
+=item *
+debug (optional, default: 0) - print debugging messages.
+
+=back
 
-=head1 SUBROUTINES/METHODS
 =cut
 
+sub connect {
+    my $self = shift; 
+
+    my $user_agent = LWP::UserAgent->new(ssl_opts => { verify_hostname => 0 });
+    my $key_request = HTTP::Request->new(GET => $self->base_uri.'type=keygen&user='.$self->username.'&password='.$self->password);
+    my $key_response = $user_agent->request($key_request);
+
+    my $pa_response = $self->_check_response_and_parse($key_response);
+
+    if ($pa_response->{status} eq 'success') {
+        $self->_api_key($pa_response->{result}->{key});
+        return 1;
+    }
+    else {
+        return 0;
+    }   
+}
+
 has host => ( is => 'ro',
                 isa => 'Str',
                 required => '1',
@@ -217,18 +276,24 @@ sub _debug_print {
 
 =head1 OBJECT METHODS 
 
-=head2 commit([vsys_id => $num], [poll => $seconds])
+=head2 commit(%parameters)
 
 The commit function commits the current candidate configuration to the Palo Alto firewall.
 
-The 'vsys_id' argument takes the virtual system ID (which ranges from [1 - 225]). If it is defined, a partial commit is performed on that vsys. i
-If no vsys is defined, the firewall performs a complete commit.
-
-The 'poll' argument takes an integer number of seconds. The script will check on the status of the commit job and return information every poll interval.
-If poll is not specified, the poll interval defaults to 5 seconds.
-
     $pa_firewall->commit(vsys_id => 10, poll => 10);
 
+=head3 Arguments
+
+=over 
+
+=item * 
+vsys_id (optional) - if supplied, performs a partial commit on the vsys specified. If not provided, performs a full commit on the device.
+
+=item *
+poll (optional, default: 5 seconds) - defines the interval (seconds) that the method will poll the device to get the current status of the commit job.
+
+=back
+
 =cut
 
 sub commit {
@@ -258,36 +323,21 @@ sub commit {
     } while ($job_result->{result}->{job}->{result} eq 'PEND');
 }
 
-=head2 connect(host => $hostname, username => $user, password => $pass, [ssl => 0|1], [debug => 0|1])
 
-This function connects to the Palo Alto with the credentials specified.
+=head2 vsys($action, %parameters)
 
-Host, username and password are mandatory.
-If not specified SSL is used, but it can be disabled using the argument ssl => 0
+=head3 Arguments
 
-Detailed debugging can be turned on using the debu => 1 argument. It is off by default.
+=over
 
-=cut
+=item * $action - perform an action: ['get' | 'set' | 'delete']
 
-sub connect {
-    my $self = shift; 
+=item * vsys_id - the ID of the virtual system to perform the action on.
 
-    my $user_agent = LWP::UserAgent->new(ssl_opts => { verify_hostname => 0 });
-    my $key_request = HTTP::Request->new(GET => $self->base_uri.'type=keygen&user='.$self->username.'&password='.$self->password);
-    my $key_response = $user_agent->request($key_request);
+=item * display-name - sets the display name of the virtual system.
 
-    my $pa_response = $self->_check_response_and_parse($key_response);
+=back
 
-    if ($pa_response->{status} eq 'success') {
-        $self->_api_key($pa_response->{result}->{key});
-        return 1;
-    }
-    else {
-        return 0;
-    }   
-}
-
-=head2 vsys($action, vsys_id => $id, display-name => $name)
 =cut
 
 sub vsys {
@@ -311,7 +361,24 @@ sub vsys {
     return $requester->(xpath => "/config/devices/entry[\@name='localhost.localdomain']/vsys/entry[\@name='vsys$vsys_id']", element => $elements);
 }
 
-=head2 vsys_import($action, vsys_id => $id, interface => [ $int1, $int2 ], virtual-router => [ $vr1, $vr2 ])
+=head2 vsys_import($action, %parameters);
+
+    $pa_firewall->vsys_import('set', vsys_id => 4, interface => [ ethernet1/1, ethernet1/2 ], virtual-router => [ default ]);
+
+=head3 Arguments
+
+=over
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vsys_id - the ID of the virtual system to perform the action on.
+
+=item * interface - an anonymous array of one or more interfaces to add to the virtual system.
+
+=item * virtual-router - an anonymous array of one or more virtual routers to add to the virtual system.
+
+=back
+
 =cut
 
 sub vsys_import {
@@ -336,7 +403,23 @@ sub vsys_import {
     return $requester->(xpath => "/config/devices/entry[\@name='localhost.localdomain']/vsys/entry[\@name='vsys$vsys_id']/import/network", element => $elements);
 }
 
-=head2 l3_subinterface($action, parent => $interface, tag => $vlan, comment => $description)
+=head2 l3_subinterface($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * parent - the parent interface of the new subinterface
+
+=item * tag - the VLAN tag to use on the sub-interface. This is also used as the logical sub-interface identifier.
+
+=item * description - a description to add to the sub-interface.
+
+=back
+
 =cut
 
 sub l3_subinterface {
@@ -355,7 +438,21 @@ sub l3_subinterface {
 }
 
 
-=head2 virtual_router($action, vr_name => $name, interface => [ $int1, $int2 ])
+=head2 virtual_router($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vr_name - the name of the virtual router to perform the action on.
+
+=item * interface - an anonymous array of one or more interfaces to add to the virtual router.
+
+=back
+
 =cut
 
 sub virtual_router {
@@ -372,7 +469,22 @@ sub virtual_router {
                         element => $elements);     
 }
 
-=head2 zone($action, vsys_id => $id, zone => $name, layer3 => [ $int1, $int2 ])
+=head2 zone($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vsys_id - the virtual system ID to which the zone is/should be a member of.
+
+=item * zone - the name of the zone to create/update/delete.
+
+=item * layer3 - an anonymous array of one or more interfaces to add to the zone.
+
+=back
+
 =cut
 
 sub zone {
@@ -389,7 +501,28 @@ sub zone {
                         element => $elements);
 }
 
-=head2 ipv4_static_route($action, vr_name => $vr_name, route_name => $rt_name, desintation => $route, nexthop => { ip-address => $ip | interface => $egress_int })
+=head2 ipv4_static_route($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vr_name - the name of the virtual router in which the static route resides.
+
+=item * route_name - the name of the route to perform the action on.
+
+=item * destination - the IPv4 destination of the route (IP/prefix)
+
+=item * nexthop - an anonymous hash specifying the next hop
+
+=item * ip-address - the next hop IP address
+
+=item * interface - the next hop interface
+
+=back
+
 =cut
 
 sub ipv4_static_route {
@@ -407,7 +540,29 @@ sub ipv4_static_route {
 }
 
 
-=head2 ipv6_static_route($action, vr_name => $vr_name, route_name => $rt_name, desintation => $route, nexthop => { ipv6-address => $ip | interface => $egress_int })
+=head2 ipv6_static_route($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vr_name - the name of the virtual router in which the static route resides.
+
+=item * route_name - the name of the route to perform the action on.
+
+=item * destination - the IPv6 destination of the route (IP/prefix)
+
+=item * nexthop - an anonymous hash specifying the next hop
+
+=item * ipv6-address - the next hop IPv6 address
+
+=item * interface - the next hop interface
+
+=back
+
 =cut
 
 sub ipv6_static_route {
@@ -424,7 +579,22 @@ sub ipv6_static_route {
                         element => $elements);
 }
 
-=head2 address($action, vsys_id => $id, name => $address_name, ip-netmask => $ipnet | ip-range => $range)
+=head2 address($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+=item * vsys_id - the vsys ID in which the resides/shall reside.
+
+=item * name - the name of the address.
+
+=item * ip-netmask - the IP/netmask combination which defines the address.
+
+=item * ip-range - an IP address range (format: 'IPstart-IPend')
+
+=back
+
 =cut
 
 sub address {
@@ -447,7 +617,23 @@ sub address {
 }
 
 
-=head2 address_group($action, vsys_id => $id, name => $group_name, member => [ $addr1, $addr2 ])
+=head2 address_group($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vsys_id - the vsys ID in which the address group resides/shall reside.A
+
+=item * name - the name of the address group.
+
+=item * member - an anonymous array of one or more addresses. These can be either address entries created with address(), or explicit IP/netmasks (e.g. 9.8.8.8/32)
+
+=back
+
 =cut
 
 sub address_group {
@@ -469,9 +655,42 @@ sub address_group {
     return $requester->(xpath => "/config/devices/entry/vsys/entry[\@name=\'$vsys\']/address-group/entry[$name]", element => $elements);
 }
 
-=head2 security_rule($action, vsys_id => $id, name => $rulename, [from => $zone_frm, to => $zone_to, source => [ $addr1, ... ], $destination => [ $addr2, ... ], application => [ $app1, ... ], source-user => [ $user1, ... ], hip-profiles => [ $profile1, ... ], action => 'allow'|'deny')
+=head2 security_rule($action, %parameters)
+
+=head3 Arguments
+
+=over
+
+
+=item * $action - perform an action: ['get' | 'set' | 'delete']
+
+=item * vsys_id - the virtual system ID of the vsys in which the security rule resides/will reside. Defaults to 1 if not supplied.
+
+=item * name - the name of the rule.
+
+=item * from - the source zone, defaults to 'any' if not supplied.
+
+=item * to - the destination zone, defaults to 'any' if not supplied.
+
+=item * source - an anonymous array of source addresses - can be addresses, address groups or explicit IP/netmask entries. Defaults to 'any' if not supplied.
+
+=item * destination - an anonymous array of destination addresses - can be addresses, address groups or explicit IP/netmask entries. Defaults to 'any' if not supplied.
+
+=item * service - an anonymous array of one or more services. Defaults to 'application-default' if not supplied.
+
+=item * appplication - an anonymous array of one or more Palo Alto applications. Defaults to 'any' if not supplied.
+
+=item * source-user - an anonymous array of one or more Palo Alto source user mappings. Defaults to 'any' if not supplied.
+
+=item * hip-profile - an anonymous array of Host Information Profiles. defaults to 'any' if not supplied.
+
+=item * action - an action for the rule, either 'allow', 'deny' or 'drop'. Defaults to 'allow' if not supplied.
+
+=back
+
 =cut
 
+
 sub security_rule {
     my $self = shift;
     my ($action, %args) = @_;
